"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var cucumber_1 = require("cucumber");
var fs_1 = require("fs");
// tslint:disable-next-line:no-var-requires
var Gherkin = require('gherkin');
// tslint:disable-next-line:no-var-requires
var EventDataCollector = require('cucumber').formatterHelpers.EventDataCollector;
var ScenarioResultFileOperations_1 = require("./ScenarioResultFileOperations");
var FormatterAdapter = /** @class */ (function () {
    function FormatterAdapter(formatterLogger) {
        this.eventBroadcaster = new events_1.EventEmitter();
        this.eventDataCollector = new EventDataCollector(this.eventBroadcaster);
        this.formatter = new cucumber_1.JsonFormatter({
            eventDataCollector: this.eventDataCollector,
            eventBroadcaster: this.eventBroadcaster,
            log: formatterLogger.log.bind(formatterLogger),
        });
    }
    FormatterAdapter.prototype.onScenarioComplete = function (jestTestResult) {
        var _this = this;
        var promises = jestTestResult.testResults.map(function (testResult) {
            var featureTitle = testResult.ancestorTitles[0];
            var scenarioTitle = testResult.title;
            return ScenarioResultFileOperations_1.loadScenarioResult(featureTitle, scenarioTitle)
                .then(function (scenarioResult) {
                return _this
                    .generateEventsFromFeatureFile(scenarioResult.featureFilePath)
                    .then(function () {
                    var sourceLocation = { uri: scenarioResult.featureFilePath, line: scenarioResult.lineNumber };
                    var key = _this.eventDataCollector.getTestCaseKey(sourceLocation);
                    var pickle = _this.eventDataCollector.pickleMap[key];
                    _this.eventBroadcaster.emit('test-case-prepared', {
                        sourceLocation: sourceLocation,
                        steps: pickle.steps.map(function (pickleStep) {
                            var location = pickleStep.locations[pickleStep.locations.length - 1];
                            var line = location.line;
                            return {
                                sourceLocation: { uri: scenarioResult.featureFilePath, line: line },
                            };
                        }),
                    });
                    pickle.steps.forEach(function (pickleStep, index) {
                        var stepResult = scenarioResult.stepResults[index];
                        var duration = (stepResult) ? (stepResult.endTime - stepResult.startTime) || 1 : 0;
                        var status = stepResult ? stepResult.error ? cucumber_1.Status.FAILED : cucumber_1.Status.PASSED : cucumber_1.Status.SKIPPED;
                        var exception = stepResult && stepResult.error ? stepResult.error : undefined;
                        _this.eventBroadcaster.emit('test-step-finished', {
                            index: index,
                            testCase: { sourceLocation: { uri: scenarioResult.featureFilePath, line: scenarioResult.lineNumber } },
                            result: {
                                duration: duration,
                                status: status,
                                exception: exception,
                            },
                        });
                    });
                    _this.eventBroadcaster.emit('test-case-finished', {
                        sourceLocation: { uri: scenarioResult.featureFilePath, line: scenarioResult.lineNumber },
                        result: { duration: testResult.duration, status: testResult.status },
                    });
                });
            });
        });
        return Promise.all(promises);
    };
    FormatterAdapter.prototype.onTestRunComplete = function () {
        this.eventBroadcaster.emit('test-run-finished');
    };
    FormatterAdapter.prototype.log = function (logText) {
        // tslint:disable-next-line:no-console
        console.log(logText);
    };
    FormatterAdapter.prototype.generateEventsFromFeatureFile = function (featureFilePath) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            fs_1.readFile(featureFilePath, 'utf8', function (error, data) {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(data);
                }
            });
        })
            .then(function (featureText) {
            var events = Gherkin.generateEvents(featureText, featureFilePath);
            events.forEach(function (event) {
                _this.eventBroadcaster.emit(event.type, event);
                if (event.type === 'pickle') {
                    _this.eventBroadcaster.emit('pickle-accepted', {
                        type: 'pickle-accepted',
                        pickle: event.pickle,
                        uri: event.uri,
                    });
                }
            });
        });
    };
    return FormatterAdapter;
}());
exports.FormatterAdapter = FormatterAdapter;
//# sourceMappingURL=FormatterAdapter.js.map